<?php
/*
 * vpn_ipsec_export_win.php
 *
 * part of pfSense (https://www.pfsense.org)
 * Copyright (c) 2020-2022 Rubicon Communications, LLC (Netgate)
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require_once("ipsec-profile.inc");
require_once("guiconfig.inc");
require_once("classes/Form.class.php");

global $config, $mobileconfig, $a_phase1, $a_phase2, $a_cert,
	$win_supported_map, $server_list, $mobile_p1, $mobile_p2;
$input_errors = array();

if (empty($mobile_p1)) {
	$input_errors[] = gettext('IPsec配置中没有移动阶段1条目。');
} elseif ($mobile_p1['iketype'] != 'ikev2') {
	$input_errors[] = gettext('移动阶段1未使用IKEv2。导出程序仅支持IKEv2。');
} elseif (substr($mobile_p1['authentication_method'], 0, 3) != 'eap') {
	$input_errors[] = gettext('移动阶段1身份验证方法不是EAP。 导出程序仅支持EAP方法。');
} else {
	/* Locate Mobile Phase 2 entries */
	$mobile_p2 = array();
	foreach ($a_phase2 as $p2) {
		if ($p2['ikeid'] == $mobile_p1['ikeid']) {
			$mobile_p2[] = $p2;
		}
	}
	if (empty($mobile_p2)) {
		$input_errors[] = gettext('IPsec配置中没有移动阶段2条目。');
	} else {
		foreach ($mobile_p2 as $mp2) {
			if (!in_array($mp2['mode'], array('tunnel', 'tunnel6'))) {
				$input_errors[] = gettext('移动阶段2条目的模式必须是"隧道"。');
				break;
			}
		}
	}
}

if ($_POST) {
	if (($_POST['server_address'] == 'Custom Hostname') &&
	    !is_hostname($_POST['server_hostname']) &&
	    !is_ipaddr($_POST['server_hostname'])) {
		$input_errors[] = gettext('自定义主机名必须是主机名或IP地址。');
	}
}


$set_errors = iex_parameter_check('windows');
if (!empty($set_errors)) {
	$input_errors = array_merge($input_errors, $set_errors);
}

$pgtitle = array(gettext("VPN"), gettext("IPsec"), gettext("IPsec导出: Windows PowerShell"));
$pglinks = array("", "@self", "@self");
$shortcut_section = "ipsec";

if ($_POST && empty($input_errors)) {
	try {
		/* Use the user-supplied VPN name or a simple default if it was empty */
		$vpn_name = (!empty($_POST['name'])) ? $_POST['name'] : "Mobile IPsec ({$config['system']['hostname']})";
		/* Ensure it's only a filename, not a path */
		$vpn_name = basename($vpn_name);

		/* If the user submitted a valid host, use it, otherwise force automatic mode */
		if (($_POST['server_address'] == 'Custom Hostname') &&
		    (is_hostname($_POST['server_hostname']) || is_ipaddr($_POST['server_hostname']))) {
			$server_address = $_POST['server_hostname'];
		} elseif (empty($_POST['server_address']) || ($_POST['server_address'] == 'Auto') ||
		    (!is_hostname($_POST['server_address']) && !is_ipaddr($_POST['server_address']))) {
			$server_address = iex_server_list(true);
		} else {
			$server_address = $_POST['server_address'];
		}

		/* Set the user cert if using EAP-TLS mode */
		$user_certref = ($mobile_p1['authentication_method'] == 'eap-tls') ? $_POST['user_certref'] : null;

		/* Export the config */
		if ($_POST['Submit'] == gettext('下载')) {
			iew_export_archive($vpn_name, $server_address, $user_certref, true);
		}
	} catch (Exception $e) {
		$input_errors[] = sprintf(gettext('不能导出IPsec VPN: %s'), $e->getMessage());
	}
}

include("head.inc");

if ($input_errors) {
	print_input_errors($input_errors);
}

$tab_array = array();
$tab_array[] = array(gettext("Apple配置文件"), false, "vpn_ipsec_profile.php");
$tab_array[] = array(gettext("Windows PowerShell"), true, "vpn_ipsec_export_win.php");
display_top_tabs($tab_array);
?>
<?php
/* User Options */
$form = new Form(false);
$section = new Form_Section('IKEv2导出设置');

/* Name - Pre-fill with "pfSense-<P1 Descr>", if P1 descr is blank use hostname */
$section->addInput(new Form_Input(
	'name',
	'VPN名称',
	'text',
	"VPN ({$config['system']['hostname']}) - " . (!empty($mobile_p1['descr']) ? $mobile_p1['descr'] : 'Mobile IPsec')
))->setHelp('客户端在其网络列表中看到的VPN名称。 ' .
		'创建下载配置文件时也会使用此名称。');

$section->addInput(new Form_Select(
	'server_address',
	'服务器地址',
	null,
	array_combine($server_list, $server_list)
))->setHelp('选择客户端使用的服务器地址。此列表由服务器证书上的SAN条目生成。 ' .
		'Windows要求服务器地址存在于服务器证书SAN列表中。');

$section->addInput(new Form_Input(
	'server_hostname',
	'自定义主机名',
	'text',
	""
))->setHelp(' \'自定义主机名\'与服务器地址二选一。 ' .
	' 客户端连接的地址，不是上面服务器的地址。');

/* For EAP-TLS, pick a specific cert */
if ($mobile_p1['authentication_method'] == 'eap-tls') {
	/* Collect user cert list from the Peer Certificate Authority */
	$tls_client_list = array();
	foreach ($a_cert as $crt) {
		if (($mobile_p1['caref'] == $crt['caref']) && !empty($crt['prv'])) {
			$tls_client_list[$crt['refid']] = $crt['descr'];
		}
	}
	asort($tls_client_list, SORT_NATURAL | SORT_FLAG_CASE);
	$section->addInput(new Form_Select(
		'user_certref',
		'TLS用户证书',
		$_POST['user_certref'],
		$tls_client_list
	))->setHelp('选择要包含在下载存档中的TLS客户端证书。');
}

$form->add($section);

$form->addGlobal(new Form_Button(
	'Submit',
	'下载',
	null,
	'fa-download'
))->addClass('btn-primary');
$form->addGlobal(new Form_Button(
	'Submit',
	'查看',
	null,
	'fa-search'
))->addClass('btn-primary');

print($form);
?>

<?php if (empty($input_errors) &&
          ($_POST['Submit'] == gettext('View'))): ?>
<textarea cols="130" rows="40">
<?= iew_export_archive($vpn_name, $server_address, $user_certref, false); ?>
</textarea>
<br/>
<?php endif; ?>

<div class="infoblock blockopen">
<?php
print_info_box(
'<p>' . gettext('此页面生成包含Windows PowerShell脚本和证书文件的存档。') . ' ' .
gettext('PowerShell脚本的命令将导入证书并在客户端工作站上设置 VPN。') . ' ' .
'<p>' . gettext('默认情况下Windows禁止运行PowerShell脚本，可以通过修改本地策略来允许执行脚本。') . ' ' .
sprintf(gettext('请参阅%1$sPowerShell执行策略文档%2$s获得更多信息。'),
	'<a href="https://go.microsoft.com/fwlink/?LinkID=135170">', '</a>') . ' ' .
gettext('如果禁用脚本，可以将命令复制并粘贴到PowerShell提示符中来执行。') . '<p>' .
'<p>' . gettext('某些命令可能需要管理员访问权限，例如导入CA 证书。') . ' ' .
gettext('可以以管理员身份在PowerShell下运行这些命令。') . '<p>' .
'<p>' . gettext('如果“移动客户端”选项卡上的“网络列表”选项处于活动状态，该脚本将包括在客户端上设置拆分隧道的参数以及用于在VPN上为移动阶段2条目中配置的网络路由的命令。') . '<p>' .
'<p>' . gettext('导出程序检查已配置的移动阶段1和阶段2条目，并尝试找到一组与Windows客户端兼容的参数。它使用它找到的第一个匹配项，因此需要在阶段1和阶段2列表中进行合理排序并选择，或者根据需要手动编辑生成的脚本。') . ' ' .
sprintf(gettext('有关兼容参数的完整列表，请参阅有关%1$sMicrosoft文档的IPsec VPN配置%2$s。'),
	'<a href="https://docs.microsoft.com/en-us/powershell/module/vpnclient/set-vpnconnectionipsecconfiguration?view=win10-ps">', '') . '<p>'
, 'info', false);
?>

</div>

<?php if ($mobile_p1['authentication_method'] == 'eap-tls'): ?>
<?= print_info_box(gettext("如果列表中缺少TLS客户端，则可能是由于IPsec对等证书颁发机构和客户端证书之间的CA不匹配，或者此防火墙上不存在客户端证书。")); ?>
<?php endif; ?>
<?php
include("foot.inc");
