<?php
/*
 * vpn_ipsec_profile.php
 *
 * part of pfSense (https://www.pfsense.org)
 * Copyright (c) 2016-2022 Rubicon Communications, LLC (Netgate)
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require_once("ipsec-profile.inc");
require_once("guiconfig.inc");
require_once("classes/Form.class.php");

global $config, $mobileconfig, $a_phase1, $a_phase2, $a_cert,
	$apple_supported_map, $server_list,
	$mobile_p1, $mobile_p2;
$input_errors = array();

$debug = true;

$profile_suffix = '.mobileconfig';

/*
 * See also: https://developer.apple.com/business/documentation/Configuration-Profile-Reference.pdf
 *           https://support.apple.com/apple-configurator
 *
 */

if (empty($mobile_p1)) {
	$input_errors[] = gettext('IPsec配置中没有移动阶段1条目。');
} else {
	/* Locate Mobile Phase 2 entries */
	if (empty($mobile_p2)) {
		$input_errors[] = gettext('IPsec配置中没有移动阶段2条目。');
	} else {
		foreach ($mobile_p2 as $mp2) {
			if (!in_array($mp2['mode'], array('tunnel', 'tunnel6'))) {
				$input_errors[] = gettext('移动阶段2条目的模式必须为"隧道"。');
				break;
			}
		}
	}
}

$valid_users = iep_get_valid_users();
asort($valid_users, SORT_NATURAL | SORT_FLAG_CASE);

if ($_POST) {
	if (($_POST['server_address'] == 'Custom Hostname') &&
	    !is_hostname($_POST['server_hostname']) &&
	    !is_ipaddr($_POST['server_hostname'])) {
		$input_errors[] = gettext('自定义主机名值必须是主机名或IP地址。');
	}

	if (!array_key_exists($_POST['vpnclient'], $valid_users)) {
		$input_errors[] = gettext('请求的客户端对此移动IPsec P1无效。');
	}
}

$set_errors = iex_parameter_check('apple');
if (!empty($set_errors)) {
	$input_errors = array_merge($input_errors, $set_errors);
}

if (!empty($set_errors)) {
	$input_errors = array_merge($input_errors, $set_errors);
}

$pgtitle = array(gettext("VPN"), gettext("IPsec"), gettext("IPsec导出：Apple配置文件"));
$pglinks = array("", "@self", "@self");
$shortcut_section = "ipsec";

if ($_POST && empty($input_errors)) {
	try {
		/* Use the user-supplied VPN name or a simple default if it was empty */
		$vpn_name = (!empty($_POST['name'])) ? $_POST['name'] : "Mobile IPsec ({$config['system']['hostname']})";
		/* Ensure it's only a filename, not a path */
		$vpn_name = basename($vpn_name);

		/* If the user submitted a valid host, use it, otherwise force automatic mode */
		if (($_POST['server_address'] == 'Custom Hostname') &&
		    (is_hostname($_POST['server_hostname']) || is_ipaddr($_POST['server_hostname']))) {
			$server_address = $_POST['server_hostname'];
		} elseif (empty($_POST['server_address']) || ($_POST['server_address'] == 'Auto') ||
		    (!is_hostname($_POST['server_address']) && !is_ipaddr($_POST['server_address']))) {
			$server_address = iex_server_list(true);
		} else {
			$server_address = $_POST['server_address'];
		}

		/* Validated client reference is in $_POST['vpnclient'] but it could be a username or certificate reference. */

		/* Set the user cert if using EAP-TLS mode */
		$user_certref = ($mobile_p1['authentication_method'] == 'eap-tls') ? $_POST['user_certref'] : null;
		// get_cert_client_id($caref, $user)
		/* Set the user cert if using EAP-TLS mode */
		$user = !empty($_POST['vpnclient']) ? $_POST['vpnclient'] : null;

		/* Export the config */
		if ($_POST['Submit'] == gettext('Download')) {
			$filename = str_replace(' ', '_', $vpn_name);
			if (!empty($user)) {
				$filename .= '.' . $user;
			}
			if (!empty($user_certref)) {
				$filename .= '.' . $user_certref;
			}
			$filename .= $profile_suffix;
			send_user_download('data', iep_generate_profile($vpn_name, $server_address, $user, $user_certref), $filename);
		}

	} catch (Exception $e) {
		$input_errors[] = sprintf(gettext('不能导出IPsec VPN: %s'), $e->getMessage());
	}
}

include("head.inc");

if ($input_errors) {
	print_input_errors($input_errors);
}

$tab_array = array();
$tab_array[] = array(gettext("Apple配置文件"), true, "vpn_ipsec_profile.php");
$tab_array[] = array(gettext("Windows PowerShell"), false, "vpn_ipsec_export_win.php");
display_top_tabs($tab_array);
?>

<?php
/* User Options */
$form = new Form(false);
$section = new Form_Section('Apple配置文件导出设置');

$section->addInput(new Form_Input(
	'name',
	'VPN名称',
	'text',
	"VPN ({$config['system']['hostname']}) - " . (!empty($mobile_p1['descr']) ? $mobile_p1['descr'] : 'Mobile IPsec')
))->setHelp('客户端在其网络列表中看到的VPN 名称。创建配置文件名时也使用此名称。');

/* Server Address - Select from server cert SAN entries or 'auto' */
$section->addInput(new Form_Select(
	'server_address',
	'服务器地址',
	null,
	array_combine($server_list, $server_list)
))->setHelp('选择客户端使用的服务器地址。 ' .
		'此列表是从服务器证书上的SAN条目生成的。 ' .
		'服务器地址必须存在于服务器证书SAN列表中。');

$section->addInput(new Form_Input(
	'server_hostname',
	'自定义主机名',
	'text',
	""
))->setHelp('\'自定义主机名\' 与服务器地址二选一。 ' .
	'客户端将连接的地址，不是上面服务器的地址。');

$section->addInput(new Form_Select(
	'vpnclient',
	'VPN客户端',
	(empty($_POST['vpnclient']) ? $_SESSION['Username'] : $_POST['vpnclient']),
	$valid_users
))->setHelp('选择要导出的客户端。根据IPsec移动P1设置，可以是用户条目或TLS证书。');

$form->add($section);

$form->addGlobal(new Form_Button(
	'Submit',
	'下载',
	null,
	'fa-download'
))->addClass('btn-primary');

$form->addGlobal(new Form_Button(
	'Submit',
	'查看',
	null,
	'fa-search'
))->addClass('btn-primary');

print($form);
?>

<?php if (empty($input_errors) &&
          ($_POST['Submit'] == gettext('View'))): ?>
<textarea cols="130" rows="40">
<?= iep_generate_profile($vpn_name, $server_address, $user, $vpnclient); ?>
</textarea>
<br/>
<?php endif; ?>

<div class="infoblock blockopen">
<?php
print_info_box(
'<p>' . gettext('此页面生成与 Apple 产品兼容的 IPsec VPN 配置文件。') . ' ' .
gettext('使用时需要将生成的配置文件导入客户端设备或工作站。') .
'<p>' . sprintf(gettext('有关创建和使用配置文件的详细信息，请访问%1$sApple配置站点%2$s。'),
	'<a href="https://support.apple.com/apple-configurator">', '</a>') . ' ' .
'<p>' . sprintf(gettext('有关配置文件的详细信息，请参阅%1$sApple配置文件参考文档%2$s。'),
	'<a href="https://developer.apple.com/business/documentation/Configuration-Profile-Reference.pdf">', '</a>')
, 'info', false);
?>

</div>

<?php if ($mobile_p1['authentication_method'] == 'eap-tls'): ?>
<?= print_info_box(gettext("如果列表中缺少TLS客户端，可能是由于IPsec对等证书颁发机构和客户端证书之间的CA不匹配，或者此防火墙上不存在客户端证书。")); ?>
<?php endif; ?>
<?php
include("foot.inc");
